﻿using System;
using System.Text;

using System.Drawing;
using System.Runtime.InteropServices;

namespace GradientionSample
{
    public sealed class Win32
    {
        [DllImport("coredll.dll", SetLastError = true, EntryPoint = "GradientFill")]
        public extern static bool GradientFill(
            IntPtr hdc, TRIVERTEX[] pVertex, uint dwNumVertex,
            GRADIENT_RECT[] pMesh, uint dwNumMesh, uint dwMode);

        public struct TRIVERTEX
        {
            public int x;
            public int y;
            public ushort Red;
            public ushort Green;
            public ushort Blue;
            public ushort Alpha;

            public TRIVERTEX(int x, int y, Color color)
                : this(x, y, color.R, color.G, color.B, color.A)
            {
            }

            public TRIVERTEX(
                int x, int y,
                ushort red, ushort green, ushort blue,
                ushort alpha)
            {
                this.x = x;
                this.y = y;
                this.Red = (ushort)(red << 8);
                this.Green = (ushort)(green << 8);
                this.Blue = (ushort)(blue << 8);
                this.Alpha = (ushort)(alpha << 8);
            }

        }
        public struct GRADIENT_RECT
        {
            public uint UpperLeft;
            public uint LowerRight;
            public GRADIENT_RECT(uint ul, uint lr)
            {
                this.UpperLeft = ul;
                this.LowerRight = lr;
            }
        }

        public enum GRADIENT_FILL : int
        {
            /// <summary>
            /// 水平方向のグラデーション
            /// </summary>
            GRADIENT_FILL_RECT_H = 0x00000000,

            /// <summary>
            /// 縦方向のグラデーション
            /// </summary>
            GRADIENT_FILL_RECT_V = 0x00000001,

            /// <summary>
            /// Windows CEではサポートされていません
            /// </summary>
            GRADIENT_FILL_TRIANGLE = 0x00000002
        }

        // 角丸の矩形を描画するのに使用
        [DllImport("coredll.dll")]
        public static extern bool RoundRect(IntPtr hdc, int nLeftRect, int nTopRect,
        int nRightRect, int nBottomRect, int nWidth, int nHeight);

        [DllImport("coredll.dll")]
        public static extern IntPtr CreateSolidBrush(uint crColor);

        [DllImport("coredll.dll")]
        public static extern IntPtr CreatePen(int fnPenStyle, int nWidth, uint crColor);

        [DllImport("coredll.dll")]
        public static extern IntPtr SelectObject(IntPtr hdc, IntPtr hgdiobj);

        [DllImport("coredll.dll")]
        public static extern bool DeleteObject(IntPtr hObject);

        public static uint RGB(Color color)
        {
            return 0x00000000 | ((uint)(color.B) << 16) | ((uint)(color.G) << 8) | (uint)(color.R);
        }
    }
}
